from .fields import MsgFields as _MsgFields
from .taskresults import TaskResults
from .taskresult import TaskResult, TaskSeed

MsgFields = _MsgFields()
TaskResults = TaskResults
TaskResult = TaskResult
TaskSeed = TaskSeed



# -*- coding: utf-8 -*-

import unittest
import colander

from ..taskresults import TaskResults
from ..taskresult import TaskResult
from ..schemas import _BaseTaskSchema


class TestMessage(unittest.TestCase):
    def setUp(self):
        self.task_name_1 = "test"

    def test_init(self):
        m = TaskResults()
        self.assertIsNotNone(m)

    def test_single_key(self):
        m = TaskResults()
        tr = TaskResult(self.task_name_1, "1.1.1", 100)
        tr.add("a", "result")
        tr.finish()
        m.add_task_result(tr)

        # now extract the test
        tr_extract = m.get_task_result(self.task_name_1, 100)
        self.assertIsNotNone(tr_extract)
        self.assertEqual(tr_extract.success, True)
        self.assertEqual(tr_extract.get_data("a"), "result")

    def test_replacement_key(self):
        # check we can overwrite an old result with a new one
        m = TaskResults()
        tr = TaskResult(self.task_name_1, "1.1.1", 100)
        tr.add("a", "result")
        tr.finish()
        m.add_task_result(tr)

        tr2 = TaskResult(self.task_name_1, "1.1.1", 100)
        tr2.add("a", "result2")
        tr2.finish()
        m.add_task_result(tr2)

        # now extract the test
        tr_extract = m.get_task_result(self.task_name_1, 100)
        self.assertIsNotNone(tr_extract)
        self.assertEqual(tr_extract.success, True)
        self.assertEqual(tr_extract.get_data("a"), "result2")

    def test_missing_task(self):
        m = TaskResults()
        try:
            m.get_task_result(self.task_name_1, 100)
        except ValueError:
            assert(True)
        else:
            assert(False)

    def test_missing_val(self):
        m = TaskResults()
        tr = TaskResult(self.task_name_1, "1.1.1", 100)
        tr.add("a", "result")
        tr.finish()
        m.add_task_result(tr)

        try:
            m.get_task_result(self.task_name_1, 999)
        except ValueError:
            assert(True)
        else:
            assert(False)

    def test_unicode(self):
        task = "unicoder"
        key = u"俺"
        val = u"鍵"

        class UnicodeSchema(_BaseTaskSchema):
            z = colander.SchemaNode(colander.String(), name=key)

        m = TaskResults()
        tr = TaskResult(task, "1.1.1", 100, schema=UnicodeSchema)
        tr.add("a", "result")
        tr.add(key, val)
        tr.finish()
        m.add_task_result(tr)

        tr_extract = m.get_task_result(task, 100, schema=UnicodeSchema)

        self.assertEqual(tr_extract.get_data(key), val)

    def test_task_loop(self):

        class TaskLoop1Schema(_BaseTaskSchema):
            y = colander.SchemaNode(colander.List(), name="y")
            z = colander.SchemaNode(colander.List(), name="z")

        class TaskLoop2Schema(_BaseTaskSchema):
            y = colander.SchemaNode(colander.List(), name="y")
            z = colander.SchemaNode(colander.List(), name="z")

        m = TaskResults()
        tr = TaskResult("task1", "1.1.1", 100, schema=TaskLoop1Schema)
        tr.add("y", [1, 2, 3])
        tr.add("z", [1, 2, 3])
        tr.finish()

        tr2 = TaskResult("task2", "1.1.1", 100, schema=TaskLoop2Schema)
        tr.add("y", "hello")
        tr.add("z", "foo")
        tr.finish()

        m.add_task_result(tr)
        m.add_task_result(tr2)

        self.assertEqual(set(m.tasks()), set(["task1", "task2"]))

    def test_task_item_loop(self):

        class TaskLoop1Schema(_BaseTaskSchema):
            y = colander.SchemaNode(colander.List(), name="y")
            z = colander.SchemaNode(colander.List(), name="z")

        class TaskLoop2Schema(_BaseTaskSchema):
            y = colander.SchemaNode(colander.List(), name="y")
            z = colander.SchemaNode(colander.List(), name="z")

        m = TaskResults()
        tr = TaskResult("task1", "1.1.1", 100, schema=TaskLoop1Schema)
        tr.add("y", [1, 2, 3])
        tr.add("z", [1, 2, 3])
        tr.finish()

        tr2 = TaskResult("task2", "1.1.1", 100, schema=TaskLoop2Schema)
        tr.add("y", "hello")
        tr.add("z", "foo")
        tr.finish()

        m.add_task_result(tr)
        m.add_task_result(tr2)

        self.assertEqual(set(m.task_objs("task1")), set([100]))

    def test_task_multi_tmoid(self):

        class TaskLoop1Schema(_BaseTaskSchema):
            y = colander.SchemaNode(colander.List(), name="y")
            z = colander.SchemaNode(colander.List(), name="z")

        class TaskLoop2Schema(_BaseTaskSchema):
            a = colander.SchemaNode(colander.String(), name="a")
            b = colander.SchemaNode(colander.String(), name="b")

        m = TaskResults()
        tr = TaskResult("task1", "1.1.1", 100, schema=TaskLoop1Schema)
        tr.add("y", [1, 2, 3])
        tr.add("z", [1, 2, 3])
        tr.finish()

        tr2 = TaskResult("task1", "1.1.1", 101, schema=TaskLoop1Schema)
        tr2.add("y", [4, 5, 6])
        tr2.add("z", [7, 8, 9])
        tr2.finish()

        tr3 = TaskResult("task2", "1.1.1", 100, schema=TaskLoop2Schema)
        tr3.add("a", "hello")
        tr3.add("b", "foo")
        tr3.finish()

        m.add_task_result(tr)
        m.add_task_result(tr2)
        m.add_task_result(tr3)

        self.assertEqual(set(m.task_objs("task1")), set([100, 101]))

        tr_extract = m.get_task_result("task1", 100, schema=TaskLoop1Schema)
        tr2_extract = m.get_task_result("task1", 101, schema=TaskLoop1Schema)
        tr3_extract = m.get_task_result("task2", 100, schema=TaskLoop2Schema)

        self.assertEqual(tr_extract.get_data("y"), [1, 2, 3])
        self.assertEqual(tr_extract.get_data("z"), [1, 2, 3])
        self.assertEqual(tr2_extract.get_data("y"), [4, 5, 6])
        self.assertEqual(tr2_extract.get_data("z"), [7, 8, 9])
        self.assertEqual(tr3_extract.get_data("a"), "hello")
        self.assertEqual(tr3_extract.get_data("b"), "foo")



# -*- coding: utf-8 -*-

from .taskresult import TaskResult


class TaskResults(dict):
    """
    JobResult is a dict for storing TaskResults with convenience functions.
    """
    def __init__(self, *args, **kwargs):
        dict.__init__(self, *args, **kwargs)

    def add_task_result(self, task_result):
        if not task_result.task_name:
            raise ValueError("TaskResult must specify a task")
        if not task_result.tmoid:
            raise ValueError("TaskResult must specify a target monitoring "
                             "object id")

        self.setdefault(task_result.task_name, {})
        self[task_result.task_name][task_result.tmoid] = dict(task_result)

    def tasks(self):
        return self.iterkeys()

    def task_objs(self, task):
        try:
            self[task]
        except KeyError:
            raise ValueError("%s task not found in message" % task)

        return self[task].iterkeys()

    def get_task_result(self, task, tmoid, schema=None):
        """
        Get a single TaskResult based on a task name and target monitoring
        ID.
        """
        try:
            self[task][tmoid]
        except KeyError:
            raise ValueError("Could not find task {0} with tmoid {1} in "
                             "results".format(task, tmoid))

        tr = TaskResult.init_from_dict(task, tmoid, self[task][tmoid], schema=schema)
        return tr



# -*- coding: utf-8 -*-

import colander
from .fields import MsgFields as F


class _MetaSchema(colander.Schema):
    start_time = colander.SchemaNode(colander.String(), name=F.META_START_TIME)
    end_time = colander.SchemaNode(colander.String(), name=F.META_END_TIME)
    err = colander.SchemaNode(colander.String(), name=F.META_ERR, missing=None)
    msg = colander.SchemaNode(colander.String(), name=F.META_MSG, missing=None)


class _BaseTaskSchema(colander.Schema):
    meta = _MetaSchema(name=F.META)


class TestSchema(_BaseTaskSchema):
    # A stable schema for testing purposes.
    a = colander.SchemaNode(colander.String(), name="a")
    b = colander.SchemaNode(colander.String(), name="b", missing=None)
    uni = colander.SchemaNode(colander.String(), name="テスト", missing=None)


# ###### Actual Task schema definitions ########

class WhoisSchema(_BaseTaskSchema):
    whois_raw = colander.SchemaNode(colander.String(), name=F.WHOIS_RAW)



# -*- coding: utf-8 -*-

# ConstantDict as detailed at
# http://loose-bits.com/2011/06/constantdict-yet-another-python.html


class ConstantDict(object):
    """An enumeration class."""
    _dict = None

    @classmethod
    def dict(cls):
        """Dictionary of all upper-case constants."""
        if cls._dict is None:
            val = lambda x: getattr(cls, x)
            cls._dict = dict(((c, val(c)) for c in dir(cls)
                             if c == c.upper()))
        return cls._dict

    def __contains__(self, value):
        return value in self.dict().values()

    def __iter__(self):
        for value in self.dict().values():
            yield value

    def __getattr__(self, name):
        if name in self.dict():
            return self._dict[name]
        else:
            return object.__getattribute__(self, name)


class MsgFields(ConstantDict):
    ASNIP = "asnip"
    ARTIFACT = "arte"
    DOMAIN = "dom"
    GEOIP = "geoip"
    HOSTNAME = "host"
    HTTP_STATUS = "httpstat"
    IP = "ip"
    PATH = "path"
    SCREENSHOT = "sshot"
    SPIDER = "spider"
    URL = "url"
    WHOIS = "whois"
    WHOIS_RAW = "raw"

    META = "_"
    META_START_TIME = "stime"
    META_END_TIME = "etime"
    META_SUCCESS = "ok"
    META_RESULT = "r"
    META_VERSION = "v"
    META_ERR = "e"
    META_MSG = "m"
